// *********************************************************************
// *                        GW-USB-06_RTCC                             *
// *                 Real-time clock demo example                      *
// *********************************************************************
// Demonstration how to use the RTCC connected to TR module inside GW-USB-06.
//
// Intended for:
//   HW: - GW-USB-06
//
//   OS: - v3.07D
//
// Description:
//   - This example uses only time bytes read/write functions.
//   - To add date bytes see comments below and MCP7940N datasheet.
//   - Remember the RTCC shares I2C bus with serial EEPROM.
//     Incorrect pin setting can cause EEPROM malfunction.
//   - Software I2C master is used.
//   - Use IQRF IDE 4.30 or higher - Terminal (Use Numbers must be checked, see Help)
//   - The RTCC is controlled using SPI commands (see below)
//
//  Set of available commands sent from IDE via SPI:
//		Read RTCC: 	'r'
//		(returns 3 time bytes, e.g. 0x17 0x10 0x54 - 17:10:54)  
//
//		Write RTCC: 'w'<hours><minutes><seconds>
//		(e.g. "w$11$22$50" -> write time: 11:22:50)
//
// *********************************************************************
//
// File:    GW-USB-06_RTCC.c
// Version: v1.00                                   Revision: 14/09/2015
//
// Revision history:
//   v1.00: 14/09/2015  First release
//
// *********************************************************************
#include "includes/template-basic.h"

// I2C *****************************************************************
#define SCL	PORTA.6				// The SCL pin
#define SDA	PORTB.6				// The SDA pin

#define SCL_OUT	TRISA.6 = 0		// Configure SCL as output
#define SCL_IN	TRISA.6 = 1		// Configure SCL as input

#define SDA_OUT	TRISB.6 = 0		// Configure SDA as output
#define SDA_IN	TRISB.6 = 1		// Configure SDA as input

#include "i2cfunctions.c"

// RTCC functions ******************************************************
void initRTCC(void);
bit readRTCC(void);
bit writeRTCC(void);
bit controlClock(bit enabled);
bit setMFP(void);
bit setVBATEN(void);

// *********************************************************************

uns8 hour, minute, second;

void APPLICATION()
{
	enableSPI();
	initRTCC();
	
	while (1)
	{
		if ( !getStatusSPI() && _SPIRX ) 		// Is anything received via SPI?
		{
			if ( _SPICRCok )
			{
				stopSPI();
				
				switch (bufferCOM[0])   		// Check what is in bufferCOM
				{	
					case 'r':                  	// Read RTCC
						if (readRTCC())
						{
							bufferCOM[0] = hour;
							bufferCOM[1] = minute;
							bufferCOM[2] = second;
						}
						else
						{
							bufferCOM[0] = 'E';
							bufferCOM[1] = 'R';
							bufferCOM[2] = 'R';
						}
						
						startSPI(3);
						pulseLEDG();
						continue;
						
					case 'w':                  	// Write RTCC
						hour = bufferCOM[1];
						minute = bufferCOM[2];
						second = bufferCOM[3];						
						writeRTCC();
						pulseLEDR();
						break;	
				}
			}
			
			startSPI(0);					// Restart SPI communication
		}
	}
}
// *********************************************************************

void initRTCC(void)
{
	setVBATEN();					// VBATEN enabled
	setMFP();						// Set MFP to 1
}
// *********************************************************************

bit readRTCC(void)
{
	i2cstart();
	i2csendbyte(0b11011110);		// Control code + write bit
	if(!i2cgetack())
	{
		i2cstop();
		return FALSE;
	}
	
	i2csendbyte(0x00);				// Address
	if(!i2cgetack())
	{
		i2cstop();
		return FALSE;
	}
	
	i2cstart();
	
	i2csendbyte(0b11011111);		// Control code + read bit
	if(!i2cgetack())
	{
		i2cstop();
		return FALSE;
	}
	
	second = i2cgetbyte();
	i2csendack();
	
	minute = i2cgetbyte();
	i2csendack();
	
	hour = i2cgetbyte();
	i2cstop();
									// Next date bytes can be included here. See MCP7940N datasheet.
									
	if ( second.7 == 0 )			// If clock is disabled
	{
		waitMS(5);
		controlClock(TRUE);			// Clock on
		return FALSE;
	}
	
	second.7 = 0;
	return TRUE;
}
// *********************************************************************

bit writeRTCC(void)
{
	i2cstart();
	i2csendbyte(0b11011110);		// Control code + write bit
	if(!i2cgetack())
	{
		i2cstop();
		return FALSE;
	}
	
	i2csendbyte(0x00);				// Address
	if(!i2cgetack())
	{
		i2cstop();
		return FALSE;
	}
	
	i2csendbyte(second | 0x80);		// Bit 7 must be set to be clock enabled
	if(!i2cgetack())
	{
		i2cstop();
		return FALSE;
	}	
	
	i2csendbyte(minute);
	if(!i2cgetack())
	{
		i2cstop();
		return FALSE;
	}
	
	i2csendbyte(hour);
	if(!i2cgetack())
	{
		i2cstop();
		return FALSE;
	}
									// Next date bytes can be included here. See MCP7940N datasheet.
	i2cstop();
	return TRUE;
}
// *********************************************************************

bit controlClock(bit enabled)
{
	i2cstart();
	i2csendbyte(0b11011110);		// Control code + write bit
	if(!i2cgetack())
	{
		i2cstop();
		return FALSE;
	}
	
	i2csendbyte(0x00);				// Address
	if(!i2cgetack())
	{
		i2cstop();
		return FALSE;
	}
	
	uns8 second = 0;
	second.7 = enabled;
	i2csendbyte(second);			// Bit 7 - clock control
	if(!i2cgetack())
	{
		i2cstop();
		return FALSE;
	}
	
	i2cstop();
	return TRUE;
}
// *********************************************************************

bit setMFP(void)
{
	i2cstart();
	i2csendbyte(0b11011110);		// Control code + write bit
	if(!i2cgetack())
	{
		i2cstop();
		return FALSE;
	}
	
	i2csendbyte(0x07);				// Address
	if(!i2cgetack())
	{
		i2cstop();
		return FALSE;
	}
	
	i2csendbyte(0b10000000);
	if(!i2cgetack())
	{
		i2cstop();
		return FALSE;
	}
	
	i2cstop();
	return TRUE;
}
// *********************************************************************

bit setVBATEN(void)
{
	i2cstart();
	i2csendbyte(0b11011110);		// Control code + write bit
	if(!i2cgetack())
	{
		i2cstop();
		return FALSE;
	}
	
	i2csendbyte(0x03);				// Address
	if(!i2cgetack())
	{
		i2cstop();
		return FALSE;
	}
	
	i2csendbyte(0b00001000);
	if(!i2cgetack())
	{
		i2cstop();
		return FALSE;
	}
	
	i2cstop();
	return TRUE;
}
// *********************************************************************
#include "i2cfunctions.c"